from unittest.runner import TextTestRunner

from .result import ColorTestResult


class ColorTestRunner(TextTestRunner):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)


	def _makeResult(self):
		return ColorTestResult(self.stream, self.descriptions, self.verbosity)



from unittest.result import TestResult

from .colors import *


class ColorTestResult(TestResult):
	separator1 = '=' * 70
	separator2 = '-' * 70
	tab = ' ' * 2

	colors = {
		'none': print,
		'func': print_yellow,
		'doc': print_cyan,
		'success': print_green,
		'fail': print_red,
		'error': print_red,
		'expected': print_green,
		'unexpected': print_red,
		'skip': print,
	}

	def __init__(self, stream, descriptions, verbosity):
		super(ColorTestResult, self).__init__(stream, descriptions, verbosity)
		self.descriptions = descriptions
		self.showAll = verbosity > 1
		self.dots = verbosity <= 1

	def printDescription(self, test, end=''):
		doc = test.shortDescription()
		if self.descriptions and doc:
			self.colors['func'](test)
			self.colors['doc'](self.tab+doc, end=end)
		else:
			self.colors['func'](test, end=end)


	def startTest(self, test):
		super(ColorTestResult, self).startTest(test)
		if self.showAll:
			self.printDescription(test)
			self.colors['none'](' ... ', end='')


	def printResult(self, short, long, color_key = 'none'):
		color = self.colors[color_key]
		if self.showAll:
			color(long)
		elif self.dots:
			color(short, end='')


	def addSuccess(self, test):
		super().addSuccess(test)
		self.printResult('.', 'OK', 'success')


	def addFailure(self, test, err):
		super().addFailure(test, err)
		self.printResult('F', 'FAIL', 'fail')


	def addError(self, test, err):
		super().addError(test, err)
		self.printResult('E', 'ERROR', 'error')


	def addExpectedFailure(self, test, err):
		super(ColorTestResult, self).addExpectedFailure(test, err)
		self.printResult('x', 'expected failure', 'expected')


	def addUnexpectedSuccess(self, test):
		super(ColorTestResult, self).addUnexpectedSuccess(test)
		self.printResult('u', 'unexpected success', 'unexpected')


	def addSkip(self, test, reason):
		super(ColorTestResult, self).addSkip(test, reason)
		self.printResult('s', 'skipped {0!r}'.format(reason), 'skip')


	def printErrors(self):
		if self.dots or self.showAll:
			self.colors['none']()
		self.printErrorList('ERROR', self.errors, 'error')
		self.printErrorList('FAIL', self.failures, 'fail')
		unexpectedSuccesses = getattr(self, 'unexpectedSuccesses', ())
		if unexpectedSuccesses:
			self.colors['none'](self.separator1, end='\n')
			for test in unexpectedSuccesses:
				self.colors['unexpected']('UNEXPECTED SUCCESS', end='')
				self.colors['none'](': ', end='')
				self.printDescription(test, end='\n')


	def printErrorList(self, flavour, errors, color_key = 'none'):
		for test, err in errors:
			self.colors['none'](self.separator1, end='\n')
			self.colors[color_key](f"{flavour}", end='')
			self.colors['none'](': ', end='')
			self.printDescription(test, end='\n')
			self.colors['none'](self.separator2, end='\n')
			self.colors['none'](err)



import sys
import os


if os.name == 'nt':
	from ._win_color_types import TextColor
	from ._win_console_utils import _set_color, _reset_color
else:
	from ._linux_color_types import TextColor
	from ._linux_console_utils import _set_color, _reset_color


def color_print(*values, **kwargs):
	sys.stdout.flush()
	if 'color' in kwargs:
		_set_color(kwargs['color'])
		kwargs.pop('color')
	print(*values, **kwargs)
	sys.stdout.flush()
	_reset_color()


def print_black(*values, **kwargs):
	color_print(*values, color=TextColor.BLACK, **kwargs)


def print_dark_blue(*values, **kwargs):
	color_print(*values, color=TextColor.DARK_BLUE, **kwargs)


def print_dark_green(*values, **kwargs):
	color_print(*values, color=TextColor.DARK_GREEN, **kwargs)


def print_dark_cyan(*values, **kwargs):
	color_print(*values, color=TextColor.DARK_CYAN, **kwargs)


def print_dark_red(*values, **kwargs):
	color_print(*values, color=TextColor.DARK_RED, **kwargs)


def print_dark_magenta(*values, **kwargs):
	color_print(*values, color=TextColor.DARK_MAGENTA, **kwargs)


def print_dark_yellow(*values, **kwargs):
	color_print(*values, color=TextColor.DARK_YELLOW, **kwargs)


def print_light_gray(*values, **kwargs):
	color_print(*values, color=TextColor.LIGHT_GRAY, **kwargs)


def print_gray(*values, **kwargs):
	color_print(*values, color=TextColor.GRAY, **kwargs)


def print_blue(*values, **kwargs):
	color_print(*values, color=TextColor.BLUE, **kwargs)


def print_green(*values, **kwargs):
	color_print(*values, color=TextColor.GREEN, **kwargs)


def print_cyan(*values, **kwargs):
	color_print(*values, color=TextColor.CYAN, **kwargs)


def print_red(*values, **kwargs):
	color_print(*values, color=TextColor.RED, **kwargs)


def print_magenta(*values, **kwargs):
	color_print(*values, color=TextColor.MAGENTA, **kwargs)


def print_yellow(*values, **kwargs):
	color_print(*values, color=TextColor.YELLOW, **kwargs)


def print_white(*values, **kwargs):
	color_print(*values, color=TextColor.WHITE, **kwargs)



class TextColor:
	BLACK = 0
	DARK_BLUE = 1	
	DARK_GREEN = 2
	DARK_CYAN = 3
	DARK_RED = 4
	DARK_MAGENTA = 5
	DARK_YELLOW = 6
	LIGHT_GRAY = 7
	GRAY = 8
	BLUE = 9
	GREEN = 10
	CYAN = 11
	RED = 12
	MAGENTA = 13
	YELLOW = 14
	WHITE = 15



from setuptools import setup, find_packages


setup(
	name='customcolor',
	version='0.0.4',
	description='Library for colored output of unit tests in the console',
	long_description=open('README.md', encoding="utf-8").read(),
	long_description_content_type='text/markdown',
	author='Kirill',
	author_email='gimpelcomp@gmail.com',
	url='https://github.com/gimpelgit/customcolor/',
	packages=find_packages(),
	install_requires=[],
	python_requires='>=3.6',
	license='MIT',
	keywords='unittest colour color output print windows linux text-decoration',
	classifiers=[
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.6',
	]
)


import ctypes


class _Coord(ctypes.Structure):
	_fields_ = [
		("X", ctypes.c_short),
		("Y", ctypes.c_short)
	]


class _SmallRect(ctypes.Structure):
	_fields_ = [
		("Left", ctypes.c_short),
		("Top", ctypes.c_short),
		("Right", ctypes.c_short),
		("Bottom", ctypes.c_short)
	]


class _ConsoleScreenBufferInfo(ctypes.Structure):
	_fields_ = [
		("dwSize", _Coord),
		("dwCursorPosition", _Coord),
		("wAttributes", ctypes.c_ushort),
		("srWindow", _SmallRect),
		("dwMaximumWindowSize", _Coord)
	]


_hConsole = ctypes.windll.kernel32.GetStdHandle(ctypes.c_uint(-11))
_original_attributes = None


def _set_color(color):
	global _original_attributes
	csbi = _ConsoleScreenBufferInfo()
	ctypes.windll.kernel32.GetConsoleScreenBufferInfo(_hConsole, ctypes.byref(csbi))
	_original_attributes = csbi.wAttributes
	background_color = _original_attributes & 0xF0
	new_color = color | background_color
	ctypes.windll.kernel32.SetConsoleTextAttribute(_hConsole, new_color)


def _reset_color():
	if _original_attributes is not None:
		ctypes.windll.kernel32.SetConsoleTextAttribute(_hConsole, _original_attributes)
